#!/usr/bin/bash
#
#

#
#$Header: /home2/reps/cvs/proj/ctys/ctys-rt/src/bin/ctys-removeVM.sh,v 1.3 2011/12/05 15:57:40 acue Exp $
#

########################################################################
#
#PROJECT:      Unified Sessions Manager
#AUTHOR:       Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com
#MAINTAINER:   Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com
#SHORT:        ctys
#CALLFULLNAME: Commutate To Your Session
#LICENCE:      GPL3
#VERSION:      01_11_023
#
########################################################################
#
#     Copyright (C) 2011 Arno-Can Uestuensoez
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
########################################################################


################################################################
#                   Begin of FrameWork                         #
################################################################


#FUNCBEG###############################################################
#
#PROJECT:
MYPROJECT="Unified Sessions Manager"
#
#NAME:
#  ctys-cloneVM.sh
#
#AUTHOR:
AUTHOR="Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com"
#
#FULLNAME:
FULLNAME="Unified Sessions Manager"
#
#CALLFULLNAME:
CALLFULLNAME="Remove VirtualMachines"
#
#LICENCE:
LICENCE=GPL3
#
#TYPE:
#  bash-script
#
#VERSION:
VERSION=01_11_023
#DESCRIPTION:
#  Utility of project ctys for generation of PM data supporting 
#  ENUMERATE. This is seperated, due to some of the data requires
#  root privileges for read operations.
#
#EXAMPLE:
#
#PARAMETERS:
#
#  refer to online help "-h" and/or "-H"
#
#OUTPUT:
#  RETURN:
#  VALUES:
#
#FUNCEND###############################################################

################################################################
#                     Global shell options.                    #
################################################################
shopt -s nullglob



################################################################
#       System definitions - do not change these!              #
################################################################
#Execution anchor
MYCALLPATHNAME=$0
MYCALLNAME=`basename $MYCALLPATHNAME`
MYCALLNAME=${MYCALLNAME%.sh}
MYCALLPATH=`dirname $MYCALLPATHNAME`

#
#If a specific library is forced by the user
#
if [ -n "${CTYS_LIBPATH}" ];then
    MYLIBPATH=$CTYS_LIBPATH
    MYLIBEXECPATHNAME=${CTYS_LIBPATH}/bin/$MYCALLNAME
else
    MYLIBEXECPATHNAME=$MYCALLPATHNAME
fi

#
#identify the actual location of the callee
#
if [ -n "${MYLIBEXECPATHNAME##/*}" ];then
    MYLIBEXECPATHNAME=${PWD}/${MYLIBEXECPATHNAME}
fi
MYLIBEXECPATH=`dirname $MYLIBEXECPATHNAME`

###################################################
#load basic library required for bootstrap        #
###################################################
MYBOOTSTRAP=${MYLIBEXECPATH}/bootstrap
if [ ! -d "${MYBOOTSTRAP}" ];then
    MYBOOTSTRAP=${MYCALLPATH}/bootstrap
    if [ ! -d "${MYBOOTSTRAP}" ];then
	echo "${MYCALLNAME}:$LINENO:ERROR:Missing:MYBOOTSTRAP=${MYBOOTSTRAP}"
	cat <<EOF  

DESCRIPTION:
This directory contains the common mandatory bootstrap functions.
Your installation my be erroneous.  

SOLUTION-PROPOSAL:
First of all check your installation, because an error at this level
might - for no reason - bypass the final tests.

If this does not help please send a bug-report.

EOF
	exit 1
    fi
fi

MYBOOTSTRAP=${MYBOOTSTRAP}/bootstrap.01.01.007.sh
if [ ! -f "${MYBOOTSTRAP}" ];then
    echo "${MYCALLNAME}:$LINENO:ERROR:Missing:MYBOOTSTRAP=${MYBOOTSTRAP}"
    cat <<EOF  

DESCRIPTION:
This file contains the common mandatory bootstrap functions required
for start-up of any shell-script within this package.

It seems though your installation is erroneous or you detected a bug.  

SOLUTION-PROPOSAL:
First of all check your installation, because an error at this level
might - for no reason - bypass the final tests.

When your installation seems to be OK, you may try to set a TEMPORARY
symbolic link to one of the files named as "bootstrap.<highest-version>".

ln -s ${MYBOOTSTRAP} bootstrap.<highest-version>

in order to continue for now. 

Be aware, that any installation containing the required file will replace
the symbolic link, because as convention the common boostrap files are
never symbolic links, thus only recognized as a temporary workaround to 
be corrected soon.

If this does not work you could try one of the other versions.

Please send a bug-report.

EOF
    exit 1
fi

###################################################
#Start bootstrap now                              #
###################################################
. ${MYBOOTSTRAP}
###################################################
#OK - utilities to find components of this version#
#available now.                                   #
###################################################

#
#set real path to install, resolv symbolic links
_MYLIBEXECPATHNAME=`bootstrapGetRealPathname ${MYLIBEXECPATHNAME}`
MYLIBEXECPATH=`dirname ${_MYLIBEXECPATHNAME}`

_MYCALLPATHNAME=`bootstrapGetRealPathname ${MYCALLPATHNAME}`
MYCALLPATHNAME=`dirname ${_MYCALLPATHNAME}`

#
###################################################
#Now find libraries might perform reliable.       #
###################################################


#current language, not really NLS
MYLANG=${MYLANG:-en}

#path for various loads: libs, help, macros, plugins
MYLIBPATH=${CTYS_LIBPATH:-`dirname $MYLIBEXECPATH`}

#path for various loads: libs, help, macros, plugins
MYHELPPATH=${MYHELPPATH:-$MYLIBPATH/help/$MYLANG}


###################################################
#Check master hook                                #
###################################################
bootstrapCheckInitialPath
###################################################
#OK - Now should work.                            #
###################################################

MYCONFPATH=${MYCONFPATH:-$MYLIBPATH/conf/ctys}
if [ ! -d "${MYCONFPATH}" ];then
    echo "${MYCALLNAME}:$LINENO:ERROR:Missing:MYCONFPATH=${MYCONFPATH}"
    exit 1
fi

if [ -f "${MYCONFPATH}/versinfo.conf.sh" ];then
    . ${MYCONFPATH}/versinfo.conf.sh
fi

MYMACROPATH=${MYMACROPATH:-$MYCONFPATH/macros}
if [ ! -d "${MYMACROPATH}" ];then
    echo "${MYCALLNAME}:$LINENO:ERROR:Missing:MYMACROPATH=${MYMACROPATH}"
    exit 1
fi

MYPKGPATH=${MYPKGPATH:-$MYLIBPATH/plugins}
if [ ! -d "${MYPKGPATH}" ];then
    echo "${MYCALLNAME}:$LINENO:ERROR:Missing:MYPKGPATH=${MYPKGPATH}"
    exit 1
fi

MYINSTALLPATH= #Value is assigned in base. Symbolic links are replaced by target


##############################################
#load basic library required for bootstrap   #
##############################################
. ${MYLIBPATH}/lib/base.sh
. ${MYLIBPATH}/lib/libManager.sh
#
#Germish: "Was the egg or the chicken first?"
#
#..and prevent real load order for later display.
#
bootstrapRegisterLib
baseRegisterLib
libManagerRegisterLib
##############################################
#Now the environment is armed, so let's go.  #
##############################################

if [ ! -d "${MYINSTALLPATH}" ];then
    ABORT=1;
    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing:MYINSTALLPATH=${MYINSTALLPATH}"
    gotoHell ${ABORT}
fi

MYOPTSFILES=${MYOPTSFILES:-$MYLIBPATH/help/$MYLANG/*_base_options} 
checkFileListElements "${MYOPTSFILES}"
if [ $? -ne 0 ];then
    ABORT=1;
    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing:MYOPTSFILES=${MYOPTSFILES}"
    gotoHell ${ABORT}
fi


################################################################
# Main supported runtime environments                          #
################################################################
#release
TARGET_OS="Linux: CentOS/RHEL(5+), SuSE-Professional 9.3"

#to be tested - coming soon
TARGET_OS_SOON="OpenBSD+FreeBSD+Linux"

#to be tested - might be almsot OK - but for now FFS
#...probably some difficulties with desktop-switching only?!
TARGET_OS_FFS="ffs."

#release
TARGET_WM="Gnome + fvwm"

#to be tested - coming soon
TARGET_WM_SOON="xfce"

#to be tested - coming soon
TARGET_WM_FORESEEN="KDE(might work now)"

################################################################
#                     End of FrameWork                         #
################################################################




##############################################
#Temporary workaround
if [ -n "${DISTREL}" ];then
    RELEASE=${DISTREL}
else
    if [ -n "${RELEASE}" ];then
	DISTREL=${RELEASE}
    fi
fi
##############################################

#
#Verify OS support
#
case ${MYOS} in
    Linux);;
    FreeBSD|OpenBSD);;
    SunOS);;
    CYGWIN);;
    *)
        printINFO 1 $LINENO $BASH_SOURCE 1 "${MYCALLNAME} is not supported on ${MYOS}"
	gotoHell 0
	;;
esac


if [ "${*}" != "${*//-X/}" ];then
    C_TERSE=1
fi
# if [ "${*}" != "${*//-V/}" ];then
#     if [ -n "${C_TERSE}" ];then
# 	echo -n ${VERSION}
#     else
# 	echo "$0: VERSION=${VERSION}"
#     fi
#     exit 0
# fi


. ${MYLIBPATH}/lib/misc.sh
. ${MYLIBPATH}/lib/help/help.sh
. ${MYLIBPATH}/lib/network/network.sh
. ${MYLIBPATH}/lib/hw/hook.sh
. ${MYLIBPATH}/lib/groups.sh


#path to directory containing the default mapping db
if [ -d "${HOME}/.ctys/db/default" ];then
    DEFAULT_DBPATHLST=${DEFAULT_DBPATHLST:-$HOME/.ctys/db/default}
fi

#path to directory containing the default mapping db
if [ -d "${MYCONFPATH}/db/default" ];then
    DEFAULT_DBPATHLST=${DEFAULT_DBPATHLST:-$HOME/conf/db/default}
fi


#Source pre-set environment from user
if [ -f "${HOME}/.ctys/ctys.conf.sh" ];then
    . "${HOME}/.ctys/ctys.conf.sh"
fi

#Source pre-set environment from installation 
if [ -f "${MYCONFPATH}/ctys.conf.sh" ];then
    . "${MYCONFPATH}/ctys.conf.sh"
fi

#system tools
if [ -f "${HOME}/.ctys/systools.conf-${MYDIST}.sh" ];then
    . "${HOME}/.ctys/systools.conf-${MYDIST}.sh"
else

    if [ -f "${MYCONFPATH}/systools.conf-${MYDIST}.sh" ];then
	. "${MYCONFPATH}/systools.conf-${MYDIST}.sh"
    else
	if [ -f "${MYLIBEXECPATH}/../conf/ctys/systools.conf-${MYDIST}.sh" ];then
	    . "${MYLIBEXECPATH}/../conf/ctys/systools.conf-${MYDIST}.sh"
	else
	    ABORT=1;
	    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing system tools configuration file:\"systools.conf-${MYDIST}.sh\""
	    printERR $LINENO $BASH_SOURCE ${ABORT} "Check your installation."
	    gotoHell ${ABORT}
	fi
    fi
fi


#
#dummy stubs for VMW avoids for now hook of CORE
#
function getVNCACCESSPORT () { return; }
function getGUESTVRAM () { return; }


#early prefetch
_ty=`echo " $* "| sed -n 's/([^)]*)//g;s/^.* -t/-t/;s/-t \([a-zA-Z0-9]*\) .*$/\1/p'|tr '[:lower:]' '[:upper:]'`
printDBG $S_BIN ${D_BULK} $LINENO $BASH_SOURCE "$FUNCNAME:_ty=$_ty"
if [ -n "${_ty// /}" ];then
    C_SESSIONTYPE="${_ty}"
else
    C_SESSIONTYPE="${C_SESSIONTYPE:-QEMU}"
    printINFO 2 $LINENO $BASH_SOURCE 1 "Setting default:C_SESSIONTYPE='${C_SESSIONTYPE:-QEMU}'"

fi
printDBG $S_BIN ${D_BULK} $LINENO $BASH_SOURCE "$FUNCNAME:C_SESSIONTYPE=$C_SESSIONTYPE"
C_SESSIONTYPE=$(echo "$C_SESSIONTYPE"| tr '[:lower:]' '[:upper:]')
C_SESSIONTYPE=${C_SESSIONTYPE// /}

#
#activate final execution checks
C_EXECLOCAL=1
case ${C_SESSIONTYPE} in
    XEN)
	if [ -d "${HOME}/.ctys" -a -d "${HOME}/.ctys/xen" ];then
	    if [ -f "${HOME}/.ctys/xen/xen.conf-${MYOS}.sh" ];then
		. "${HOME}/.ctys/xen/xen.conf-${MYOS}.sh"
	    fi
	fi

	if [ -d "${MYCONFPATH}/xen" ];then
	    if [ -f "${MYCONFPATH}/xen/xen.conf-${MYOS}.sh" ];then
		. "${MYCONFPATH}/xen/xen.conf-${MYOS}.sh"
	    fi
	fi
	if [ "$_XEN_INIT_OK_" != 0 ];then
	    printERR $LINENO $BASH_SOURCE 1 "Could not load \"xen.conf-${MYOS}.sh\"  for:C_SESSIONTYPE=${C_SESSIONTYPE}"
	    gotoHell ${ABORT}	    
        fi
	. ${MYLIBPATH}/lib/libXENbase.sh
	;;
    QEMU)
	if [ -d "${HOME}/.ctys" -a -d "${HOME}/.ctys/qemu" ];then
	    if [ -f "${HOME}/.ctys/qemu/qemu.conf-${MYOS}.sh" ];then
		. "${HOME}/.ctys/qemu/qemu.conf-${MYOS}.sh"
	    fi
	fi
	if [ -d "${MYCONFPATH}/qemu" ];then
	    if [ -f "${MYCONFPATH}/qemu/qemu.conf-${MYOS}.sh" ];then
		. "${MYCONFPATH}/qemu/qemu.conf-${MYOS}.sh"
	    fi
	fi
	if [ "$_QEMU_INIT_OK_" != 0 ];then
	    printERR $LINENO $BASH_SOURCE 1 "Could not load \"qemu.conf-${MYOS}.sh\"  for:C_SESSIONTYPE=${C_SESSIONTYPE}"
	    gotoHell ${ABORT}	    
        fi
	. ${MYLIBPATH}/lib/libQEMUbase.sh
	;;
    VMW)
	if [ -d "${HOME}/.ctys" -a -d "${HOME}/.ctys/vmw" ];then
	    if [ -f "${HOME}/.ctys/vmw/vmw.conf-${MYOS}.sh" ];then
		. "${HOME}/.ctys/vmw/vmw.conf-${MYOS}.sh"
	    fi
	fi

	if [ -d "${MYCONFPATH}/vmw" ];then
	    if [ -f "${MYCONFPATH}/vmw/vmw.conf-${MYOS}.sh" ];then
		. "${MYCONFPATH}/vmw/vmw.conf-${MYOS}.sh"
	    fi
	fi
	if [ "$_VMW_INIT_OK_" != 0 ];then
	    printERR $LINENO $BASH_SOURCE 1 "Could not load \"vmw.conf-${MYOS}.sh\"  for:C_SESSIONTYPE=${C_SESSIONTYPE}"
	    gotoHell ${ABORT}	    
        fi
	. ${MYLIBPATH}/lib/libVMWserver2.sh
	. ${MYLIBPATH}/lib/libVMWconf.sh
	;;

    VBOX)
	if [ -d "${HOME}/.ctys" -a -d "${HOME}/.ctys/vbox" ];then
	    if [ -f "${HOME}/.ctys/vbox/vbox.conf-${MYOS}.sh" ];then
		. "${HOME}/.ctys/vbox/vbox.conf-${MYOS}.sh"
	    fi
	fi

	if [ -d "${MYCONFPATH}/vbox" ];then
	    if [ -f "${MYCONFPATH}/vbox/vbox.conf-${MYOS}.sh" ];then
		. "${MYCONFPATH}/vbox/vbox.conf-${MYOS}.sh"
	    fi
	fi
	if [ "$_VBOX_INIT_OK_" != 0 ];then
	    printERR $LINENO $BASH_SOURCE 1 "Could not load \"vbox.conf-${MYOS}.sh\"  for:C_SESSIONTYPE=${C_SESSIONTYPE}"
	    gotoHell ${ABORT}	    
        fi
	. ${MYLIBPATH}/lib/libVBOXbase.sh
	. ${MYLIBPATH}/lib/libVBOX.sh
	. ${MYLIBPATH}/lib/libVBOXconf.sh
	;;
    *)
	ABORT=1;
	printERR $LINENO $BASH_SOURCE ${ABORT} "Sessiontype not supported:C_SESSIONTYPE=${C_SESSIONTYPE}"
	gotoHell ${ABORT}
	;;
esac


#########################################################################
#plugins - project specific commons and feature plugins                 #
#########################################################################
PLUGINPATHS=${MYINSTALLPATH}/plugins/CORE
LD_PLUGIN_PATH=${LD_PLUGIN_PATH}:${PLUGINPATHS}
MYROOTHOOK=${MYINSTALLPATH}/plugins/hook.sh
if [ ! -f "${MYROOTHOOK}" ];then 
    ABORT=2
    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing p
    ackages hook: hook=${MYROOTHOOK}"
    gotoHell ${ABORT}
fi
. ${MYROOTHOOK}
initPackages "${MYROOTHOOK}"


################################################################
#    Default definitions - User-Customizable  from shell       #
################################################################


#
#Load default values for configuration.
#
#Source pre-set environment from user
if [ -f "${HOME}/.ctys/ctys-createConfVM.d/defaults-${C_SESSIONTYPE}.ctys" ];then
    . "${HOME}/.ctys/ctys-createConfVM.d/defaults-${C_SESSIONTYPE}.ctys"
fi

#Source pre-set environment from installation 

if [ -f "${MYCONFPATH}/ctys-createConfVM.d/defaults-${C_SESSIONTYPE}.ctys" ];then
    . "${MYCONFPATH}/ctys-createConfVM.d/defaults-${C_SESSIONTYPE}.ctys"
else
    printWNG 1 $LINENO $BASH_SOURCE ${ABORT} "Cannot load system defaults, just check it."
fi


VBOX_MACHINE_ROOT="${VBOX_DEFAULT_MACHINE_ROOT}"
C_STRUCT=0;

_ARGS=;
_ARGSCALL=$*;
_RUSER0=;
LABEL=;
OLABEL=;

_CREATEIMAGE=;
_NOCREATEIMAGE=;
_SAVEPARAKERN=;
_NOSAVEPARAKERN=;

_IP=;
_TCP=;
_MAC=;

while [ -n "$1" ];do
    printDBG $S_LIB ${D_BULK} $LINENO $BASH_SOURCE "$FUNCNAME:\${1}=<${1}>"
    case $1 in
	'-t')shift;C_SESSIONTYPE=$1;C_SESSIONTYPE=$(echo "$C_SESSIONTYPE"| tr '[:lower:]' '[:upper:]');;

	'--ctys-standard-structure')C_STRUCT=0;;
	'--no-ctys-standard-structure')unset CTYS_STRUCT;;


# 	'--create-image')_CREATEIMAGE=1;;
# 	'--no-create-image')_NOCREATEIMAGE=1;;

# 	'--save-para-kernel')_SAVEPARAKERN=1;;
# 	'--no-save-para-kernel')_NOSAVEPARAKERN=1;;

        '--label='*) LABEL=${1#*=};;
        '--label-old='*) OLABEL=${1#*=};;

        '--configuration-file='*)DEFAULTSFILE=${1#*=};;

	'--target-directory='*)
	    _RDIRECTORYBASE=${1#*=};
	    _RDIRECTORYBASE=$(echo "${_RDIRECTORYBASE}"|sed 's@/[^/]\+/\.\.@@g');
	    ;;

#         '--ip='*)_IP=${1#*=};;
#         '--tcp='*)_TCP=${1#*=};;
#         '--mac='*)
# 	    _MAC=${1#*=};
# 	    _MAC="${_MAC//:/}"
# 	    _MAC=$(echo "$_MAC"|tr '[:lower:]' '[:upper:]')
# 	    if [ -n "${_MAC//[0-9A-F]/}" ];then
# 		printERR $LINENO $BASH_SOURCE 1 "Invalid MAC address:\"${1}\"."
# 		gotoHell 1
# 	    fi
# 	    ;;

        '--uuid='*)_UUID=${1#*=};;

#         '--vm-state='*)_VMSTATE=${1#*=};;

	'-f')C_FORCE=1;;

	'-H'|'--helpEx'|'-helpEx')shift;_HelpEx="${1:-$MYCALLNAME}";;
	'-h'|'--help'|'-help')_showToolHelp=1;;
	'-V')_printVersion=1;;
	'-X')C_TERSE=1;;

	'-d')shift;;


	*)
	    ABORT=1;
	    printERR $LINENO $BASH_SOURCE ${ABORT} "Unknown option \"$1\""
	    gotoHell ${ABORT}
	    ;;
    esac
    shift
done


#
#label
#
if [ -z "${LABEL}" ];then
    ABORT=1;
    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing option \"--label\"."
    gotoHell ${ABORT}
fi


#
#target directory
#
if [ -n "${_RDIRECTORYBASE}" ];then
    if [ ! -d "${_RDIRECTORYBASE}" ];then
	ABORT=1;
	printERR $LINENO $BASH_SOURCE ${ABORT} "Missing parent directory for target:\"${_RDIRECTORYBASE}\""
	gotoHell ${ABORT}
    else
	_RDIRECTORY="${_RDIRECTORYBASE}/${LABEL}"
	if [ -d "${_RDIRECTORY}" ];then
	    if [ -z "$C_FORCE" ];then
		ABORT=1;
		printERR $LINENO $BASH_SOURCE ${ABORT} "Target directory exists:\"${_RDIRECTORY}\""
		gotoHell ${ABORT}
	    fi
	else
	    mkdir "${_RDIRECTORY}"
	fi

	if [ ! -d "${_RDIRECTORY}" ];then
	    ABORT=1;
	    printERR $LINENO $BASH_SOURCE ${ABORT} "Cannot create target directory:\"${_RDIRECTORY}\""
	    gotoHell ${ABORT}
	fi
    fi
else
    ABORT=1;
    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing option \"--directory\"."
    gotoHell ${ABORT}
fi


###############################
###############################
function getProgress {
    ls -l "$1" "$2"|awk 'BEGIN{a=0;x1=0;x2=0;}$5!~/^$/{if(x1==0){x1=$5;}else{x2=$5;}}END{x=100;if(x1!=0){x=int(100*x2/x1)};print x1":"x2":"x;}'
}

function drawLine {
    [ -b "$C_TERSE" ]&&return
    echo
    setFontAttrib FRED   "."
    setFontAttrib FGREEN "."
    setFontAttrib FBKUE  "."
    setFontAttrib FRED   "."
    setFontAttrib FGREEN "."
    setFontAttrib FBKUE  "."
    setFontAttrib FRED   "."
    setFontAttrib FGREEN "."
    setFontAttrib FBKUE  "."
    setFontAttrib FRED   "."
    setFontAttrib FGREEN "."
    setFontAttrib FBKUE  "."
    setFontAttrib FRED   "."
    setFontAttrib FGREEN "."
    setFontAttrib FBKUE  "."
    setFontAttrib FRED   "."
    setFontAttrib FGREEN "."
    setFontAttrib FBKUE  "."
    setFontAttrib FRED   "."
    setFontAttrib FGREEN "."
    setFontAttrib FBKUE  "."
    echo
}

function drawStart {
    [ -b "$C_TERSE" ]&&return
    echo
    setFontAttrib BCYAN "                                                                                 "
    echo
    setFontAttrib BCYAN "  +---------------------------------------------------------------------------+  "
    echo
    setFontAttrib BCYAN "  |                                                                           |  "
    echo
    setFontAttrib BCYAN "  |                          ";
    setFontAttrib BCYAN $(printf "Start duplication of VM");
    setFontAttrib BCYAN "                          |  ";
    echo
    setFontAttrib BCYAN "  |                             Version:${VERSION//_/.}                             |  "
    echo
    setFontAttrib BCYAN "  |                                                                           |  "
    echo
    setFontAttrib BCYAN "  +---------------------------------------------------------------------------+  "
    echo
    setFontAttrib BCYAN "                                                                                 "
    echo
    echo
}


function drawCopy {
    [ -b "$C_TERSE" ]&&return
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    setFontAttrib BYELLOW "               ";
    setFontAttrib BYELLOW $(printf "Copy machine by "&&setFontAttrib BOLD  "cp");
    setFontAttrib BYELLOW "                      ";
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    echo
}

function drawCopyFinished {
    [ -b "$C_TERSE" ]&&return
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    setFontAttrib BYELLOW "                ";
    setFontAttrib BYELLOW $(printf "Copy machine finished.");
    setFontAttrib BYELLOW "                 ";
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    echo
}

function drawRegister {
    [ -b "$C_TERSE" ]&&return
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    setFontAttrib BYELLOW "                ";
    setFontAttrib BYELLOW $(printf "Register machine.");
    setFontAttrib BYELLOW "                      ";
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    echo
}

function drawRegisterFinished {
    [ -b "$C_TERSE" ]&&return
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    setFontAttrib BYELLOW "             ";
    setFontAttrib BYELLOW $(printf "Register machine finished.");
    setFontAttrib BYELLOW "                ";
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    echo
}

function drawAdapt {
    [ -b "$C_TERSE" ]&&return
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    setFontAttrib BYELLOW "       ";
    setFontAttrib BYELLOW $(printf "Adapt attributes by "&&setFontAttrib BOLD  "ctys-createConfVM.sh");
    setFontAttrib BYELLOW "        ";
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    echo
}

function drawAdaptFinished {
    [ -b "$C_TERSE" ]&&return
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    setFontAttrib BYELLOW "    ";
    setFontAttrib BYELLOW $(printf "Adapt attributes finished "&&setFontAttrib BOLD  "ctys-createConfVM.sh");
    setFontAttrib BYELLOW "     ";
    echo
    setFontAttrib BYELLOW "                                                       "
    echo
    echo
}

###############################
###############################


#
#For: generic
#
function removeVM () {
    local _fileset=;
    local _UNREGISTERLABEL="UNTERGISTER"

    local XML=${VBOX_MACHINE_ROOT}/${LABEL}/${LABEL}.xml
    
    function mkUNREGISTERED () {
	echo "DATETIME=$DATETIME">"${1}/$_UNREGISTERLABEL"
	echo "CLONE-VERSION=$VERSION">>"${1}/$_UNREGISTERLABEL"
	echo "UID=$MYUID">>"${1}/$_UNREGISTERLABEL"
	echo "GUID=$MYGID">>"${1}/$_UNREGISTERLABEL"
	echo "HOST=$MYHOST">>"${1}/$_UNREGISTERLABEL"
	echo "OS=$MYOS">>"${1}/$_UNREGISTERLABEL"
	echo "OSREL=$MYOSREL">>"${1}/$_UNREGISTERLABEL"
	echo "DIST=$MYDIST">>"${1}/$_UNREGISTERLABEL"
	echo "DISTREL=$MYREL">>"${1}/$_UNREGISTERLABEL"

	echo "SOURCE=$OLABEL">>"${1}/$_UNREGISTERLABEL"
    }

    mkCREATED "${_RDIRECTORY}"

    #ctys-conf
    if [ -f "${OLABEL}.ctys" ];then
	_ADAPT=;
	_ADAPT="${_ADAPT}  ${MYLIBEXECPATH}/ctys-attribute.sh"	
	_ADAPT="${_ADAPT}   --attribute-create=top"
	_ADAPT="${_ADAPT}   --attribute-keyonly"
	_ADAPT="${_ADAPT}   --attribute-name='#@#MAGICID-IGNORE'"
	_ADAPT="${_ADAPT}   '${_RDIRECTORY}/${LABEL}.ctys'"
	eval ${_ADAPT}
	[ -z "$C_TERSE" ]&&echo -n "Deactivated:"&&setFontAttrib BOLD "${LABEL}.ctys"
    fi



    case "${C_SESSIONTYPE}" in
	QEMU)
	    ;;

	VMW)
	    ;;

	VBOX)
	    #
	    #Fetch old config
	    #
	    local _oxml=$(${VBOXMGR} showvminfo ${LABEL}  --machinereadable|sed -n 's/CfgFile="\(.*\)"/\1/p')
	    if [ -f "${_oxml}" ];then
		[ -z "$C_TERSE" ]&&echo -n "Config:"&&setFontAttrib BOLD "${_oxml}"&&echo
	    else
		ABORT=1;
		printERR $LINENO $BASH_SOURCE ${ABORT} "Missing configuration file for VBOX:\"${OLABEL}\""
		printERR $LINENO $BASH_SOURCE ${ABORT} "Only registered VMs could be cloned for VBOX."
		gotoHell ${ABORT}
	    fi
	    echo

	    #
	    #deregister and detach existing VMs
	    #
	    local _unreg=$(${VBOXMGR} unregistervm $LABEL|grep 'medium attachments');
	    if [ "$_nx" != "${_nx#ERROR:}" -o -n "$_unreg" ];then
		local _cnt=$(
		    ${VBOXMGR} showvminfo  $LABEL --machinereadable|awk -F'=' -v mgr="${VBOXMGR}" -v l="$LABEL"  '
                       /storagecontrollername/{
                          if(devices!~/^$/){devices=devices";"mgr" storagectl "l" --name "$2" --remove";}
                          else{devices=mgr" storagectl "l" --name "$2" --remove";}
                          next;
                       }
                       END{print devices;}
                       '
		    );
		if [ -n "$_cnt" ];then
		    printFINALCALL 0  $LINENO $BASH_SOURCE "VM detach storage" "${_cnt}"
		    eval $_cnt
		fi
		${VBOXMGR} unregistervm $LABEL;
	    fi

	    #
	    #adjust storage
	    #
	    local _idx=0;
	    for i in ${_RDIRECTORY}/*.vdi;do
		${VBOXMGR} closemedium disk $i;
	    done


            #
            #attach storage
            #
            #->limited implementations, missing:
            #  ...
	    call=$(cat ${XML}| awk -F'=' -v vm="${LABEL}" '
                   BEGIN{cn=0;at=0;call="";call0="";}
                   /<StorageController .*[^\/]>/{
                     cn+=1;
                   }
     
                   /<AttachedDevice .*[^\/]>/{
                     at+=1;
                   }
     
                   /<StorageController /{
                     x2=$0;gsub("^.*name=\"","",x2);gsub("\".*$","",x2);
                     if(x2!~/^$/){call0=call0" --storagectl \""x2"\" ";}
     
                     x3=$0;gsub("^.*type=\"","",x3);gsub("\".*$","",x3);
                     x4=$0;gsub("^.*PortCount=\"","",x4);gsub("\".*$","",x4);
                     x2="";x3="";x4="";
                   }
     
                   /<AttachedDevice /{
                     x2=$0;gsub("^.*device=\"","",x2);gsub("\".*$","",x2);
                     call0=call0"--device \""x2"\"";
     
                     x3=$0;gsub("^.*type=\"","",x3);gsub("\".*$","",x3);
                     if(x3~/DVD/){x3="dvddrive"};
                     if(x3~/HardDisk/){x3="hdd"};
                     if(x3~/Floppy/){x3="fdd"};
                     if(x3!~/^$/){call0=call0" --type=\""x3"\" ";}
                     x4=$0;gsub("^.*port=\"","",x4);gsub("\".*$","",x4);
                     call0=call0" --port \""x4"\"";
                     x2="";x3="";x4="";
                   }
     
                   /<Image /{
                     x2=$0;gsub("^.*uuid=\"","",x2);gsub("\".*$","",x2);gsub("[{}]","",x2);
                     image="\""x2"\" ";
                     x2="";x3="";x4="";
     
                   }
                   /<\/AttachedDevice>/{
                     at-=1;
                   }
                   /<\/StorageController>/{
                     cn-=1;
                     if(cn==0){
                        if(image!~/^$/){
                          image=" --medium "image
                          call=call"${VBOXMGR} storageattach  "vm" "call0" "image";";
                        }else{
                          image=" --medium emptydrive" 
                          call=call"${VBOXMGR} storageattach  "vm" "call0" "image";";
                        }
                        call0="";
                        image="";
                     }
                   }
     
                   END{
                        print call;
                   }
                 ' 
		)
	    printFINALCALL 0  $LINENO $BASH_SOURCE "VM attach storage" "${call}"
	    eval $call

	    drawRegisterFinished
	    ;;

	XEN)
	    ;;

    esac








    case "${C_SESSIONTYPE}" in
	QEMU)
	    ;;

	VMW)
	    if [ -f "${OLABEL}.vmx" ];then
		drawLine
		${CP}  ${CPR} "${OLABEL}.vmx"      "${_RDIRECTORY}/${LABEL}.vmx"
		cp2oldDeactivate "${OLABEL}.vmx" "${_RDIRECTORY}"
		[ -z "$C_TERSE" ]&&echo -n "Copied:${OLABEL}.vmx -> "&&setFontAttrib BOLD "${LABEL}.vmx"
	    else
		ABORT=1;
		printERR $LINENO $BASH_SOURCE ${ABORT} "Missing configuration file for VMW:\"${OLABEL}.VMX\""
		gotoHell ${ABORT}
	    fi
	    ;;

	XEN)
	    if [ -f "${OLABEL}.conf" ];then
		drawLine
		${CP}  ${CPR} "${OLABEL}.conf"      "${_RDIRECTORY}/${LABEL}.conf"
		cp2oldDeactivate "${OLABEL}.conf" "${_RDIRECTORY}"
		[ -z "$C_TERSE" ]&&echo -n "Copied:${OLABEL}.conf -> "&&setFontAttrib BOLD "${LABEL}.conf"
	    else
		ABORT=1;
		printERR $LINENO $BASH_SOURCE ${ABORT} "Missing configuration file for XEN:\"${OLABEL}.conf\""
		gotoHell ${ABORT}
	    fi
	    ;;
    esac

    ######
    ##########
    ######
    drawLine
    drawAdapt
    _ADAPT=;
    if [ -n "$_IP" ];then
	_ADAPT="${_ADAPT}  IP=$_IP"
    fi

    if [ -n "$_MAC" ];then
	_ADAPT="${_ADAPT}  MAC=$_MAC"
    fi

    if [ -n "$_UUID" ];then
	_ADAPT="${_ADAPT}  UUID=$_UUID"
    fi

    if [ -n "$_VMSTATE" ];then
	_ADAPT="${_ADAPT}  VMSTATE=$_VMSTATE"
    fi

    _ADAPT="${_ADAPT}  ${MYLIBEXECPATH}/ctys-createConfVM.sh"
    _ADAPT="${_ADAPT}  -t ${C_SESSIONTYPE}"
    _ADAPT="${_ADAPT}  --label=${LABEL}"
    _ADAPT="${_ADAPT}  --defaults-file=${_ORIGIN}/${OLABEL}.defaults"
    _ADAPT="${_ADAPT}  --no-create-image"
    _ADAPT="${_ADAPT}  --no-save-para-kernel"
    _ADAPT="${_ADAPT}  --auto-all"
    _ADAPT="${_ADAPT}  ${C_TERSE:+ -X}"
    cd "${_RDIRECTORY}"
    printFINALCALL 0  $LINENO $BASH_SOURCE "VM-Configuration" "${_ADAPT}"
    eval "${_ADAPT}"
    if [ $? -ne 0 ];then
	ABORT=1;
	printERR $LINENO $BASH_SOURCE ${ABORT} "Failed to create new configuration from:<$OLABEL>"
	gotoHell ${ABORT}
    fi
    cd -
    drawAdaptFinished
   #################
  #################
 #################

    drawLine
    if [ -z "$C_TERSE" ]; then 
	echo
	echo -n "Finished, duration was:"&&getDiffTime $_c0
	echo
	echo
    fi
}


#
#For: QEMU
#
function copyQEMU () {
    copyVM $*
}


#
#For: VBOX
#
#current version supports local '*vdi' only.
function copyVBOX () {
    if [ -n "${C_STRUCT}" ];then
	VBOX_MACHINE_ROOT="${_RDIRECTORY%/*}"
    fi
    copyVM $*
}


#
#For: VMW
#
function copyVMW () {
    copyVM $*
}


#
#For: XEN
#
function copyXEN () {
    copyVM $*
}



#start time
_c0=$(getCurTime);

case ${C_SESSIONTYPE} in
    QEMU)
	copyVM
	if [ -z "$C_TERSE" ]; then 
	    echo
	    echo "Finished, may work now from the box."
	    echo -n "Duration was:"&&getDiffTime $_c0
	    echo
	    echo
	fi
	printINFO 2 $LINENO $BASH_SOURCE 1 "Finished, may work now from the box."
	;;

    XEN)
	copyXEN
	if [ -z "$C_TERSE" ]; then 
	    echo
	    echo "Finished, may work now from the box."
	    echo -n "Duration was:"&&getDiffTime $_c0
	    echo
	    echo
	fi
	printINFO 2 $LINENO $BASH_SOURCE 1 "Finished, may work now from the box."
	;;

    VMW)
	copyVMW
	if [ -z "$C_TERSE" ]; then 
	    echo
	    echo "Finished, may work now from the box."
	    echo -n "Duration was:"&&getDiffTime $_c0
	    echo
	    echo
	fi
	A=1;
	printWNG 1 $LINENO $BASH_SOURCE ${A} "$(setFontAttrib FRED 'ATTENTION:')"
	printWNG 1 $LINENO $BASH_SOURCE ${A} "For $(setFontAttrib BOLD 'current CTYS version') this sessiontype $(setFontAttrib BOLD 'VMW') requires "
	printWNG 1 $LINENO $BASH_SOURCE ${A} "manual postprocessing by OEM tools of VMware(TM)."
	printWNG 1 $LINENO $BASH_SOURCE ${A} "Edit the \"<${LABEL}>.$(setFontAttrib BOLD 'vmx')\" file and adapt the values"
	printWNG 1 $LINENO $BASH_SOURCE ${A} "for:"
	printWNG 1 $LINENO $BASH_SOURCE ${A} " -> eth[0-9]"
	printWNG 1 $LINENO $BASH_SOURCE ${A} " -> uuid"
	printWNG 1 $LINENO $BASH_SOURCE ${A} " -> DisplayName"
	printWNG 1 $LINENO $BASH_SOURCE ${A} "More as required..."
	printWNG 1 $LINENO $BASH_SOURCE ${A} "Additional configuration and registration may be required for specific"
	printWNG 1 $LINENO $BASH_SOURCE ${A} "versions..."
	;;

    VBOX)
	copyVBOX
	if [ -z "$C_TERSE" ]; then 
	    echo
	    echo "Finished, may work now from the box."
	    echo -n "Duration was:"&&getDiffTime $_c0
	    echo
	    echo
	fi

	if [ -z "$_IP" -o -z "$_TCP" -o -z "$_MAC" ];then
	    A=1;
	    printINFO 1 $LINENO $BASH_SOURCE ${A} "$(setFontAttrib BOLD $(setFontAttrib FBLUE 'ATTENTION:'))"
	    printINFO 1 $LINENO $BASH_SOURCE ${A} "For $(setFontAttrib BOLD 'current CTYS version') this sessiontype $(setFontAttrib BOLD 'VBOX') requires "
	    printINFO 1 $LINENO $BASH_SOURCE ${A} "probably some manual postprocessing by OEM tools of"
	    printINFO 1 $LINENO $BASH_SOURCE ${A} "VirtualBox(TM)."
	    printINFO 1 $LINENO $BASH_SOURCE ${A} "Configure your VM in the GUI or by \"${VBOXMGR}\":"
	    if [ -z "$_MAC" ];then
		printINFO 1 $LINENO $BASH_SOURCE ${A} " -> Set $(setFontAttrib FRED 'MAC') ethernet address as required."
	    fi
	    if [ -z "$_IP" ];then
		printINFO 1 $LINENO $BASH_SOURCE ${A} " -> Set $(setFontAttrib FBLUE 'IP') address as required."
	    fi
	    if [ -z "$_TCP" ];then
		printINFO 1 $LINENO $BASH_SOURCE ${A} " -> Set $(setFontAttrib FBLUE 'TCP') address-name as required."
	    fi
	    printINFO 1 $LINENO $BASH_SOURCE ${A} " -> Configure $(setFontAttrib FBLUE 'network connection type'), e.g. Bridged, or NAT, ..."
	    printINFO 1 $LINENO $BASH_SOURCE ${A} " -> Enable $(setFontAttrib FBLUE 'RDP') access as required..."

	    printINFO 1 $LINENO $BASH_SOURCE ${A} "More as required$(setFontAttrib FRED '...')"
	fi
	;;

    *)
	ABORT=1;
	printERR $LINENO $BASH_SOURCE ${ABORT} "Sessiontype not supported:C_SESSIONTYPE=${C_SESSIONTYPE}"
	gotoHell ${ABORT}
	;;

esac


