#!/usr/bin/bash
#
#

#
#$Header: /home2/reps/cvs/proj/ctys/ctys-rt/src/lib/geometry/getScreenOffset.sh,v 1.4 2011/12/07 07:32:43 acue Exp $
#


########################################################################
#
#PROJECT:      Unified Sessions Manager
#AUTHOR:       Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com
#MAINTAINER:   Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com
#SHORT:        ctys
#CALLFULLNAME: Commutate To Your Session
#LICENCE:      GPL3
#VERSION:      01_11_023
#
########################################################################
#
# Copyright (C) 2007,2008 Arno-Can Uestuensoez
#
########################################################################


 _myLIBNAME_geometry="${BASH_SOURCE}"
# _myLIBVERS_geometry="01.11.023"
# libManInfoAdd "${_myLIBNAME_geometry}" "${_myLIBVERS_geometry}"

export _myLIBBASE_geometry="`dirname ${_myLIBNAME_geometry}`"


#FUNCBEG###############################################################
#NAME:
#  getScreenOffset
#
#TYPE:
#  bash-function
#
#DESCRIPTION:
#  Reads by default from "/etc/X11/xorg.conf" or from any other when 
#  given, the Screen-Label, xoffset, and yoffset for given ServerLayout
#  and screen, where screen could be given by it's index or label.
#
#ATTENTION:
#  SPECIAL REQUIREMENT:
#   Current implementation requires "Identifier" as first
#   entry in LayoutSections
#
#EXAMPLE:
#
#  XORGCFG=${1:-$XORGCFG_DEFAULT}
#  DEBUG=1
#
#  echo
#  echo "Layout[all] Screen0 -> $XORGCFG"
#  getScreenOffset $DEBUG "Layout[all]" "Screen0" 0 $XORGCFG
#  echo "ret=$?"
#  echo
#
#PARAMETERS:
#
# $1: DEBUG:        0=on, else off
# $2: SECTIONNAME:  Literal name of LayoutSection to use, if empty first is taken.
# $3: SCREENALIAS:  Literal name of Screen section (used in LayoutSection), if
#                   empty SCREENNR is used
# $4: SCREENNR:     Index number of screen, if SCREENALIAS given, this will be 
#                   ignored. Range is (0<=N)
# $5: XORGCFG:      Pathname to be used as configuration file for Xorg data, 
#                   if empty default is used(XORGCFG_DEFAULT).
#
#OUTPUT:
#  RETURN:
#
#  VALUES:
#    "%s %d %d", $3, $4, $5
#      $3: Screen label from xorg.conf
#      $4: xoffset
#      $5: yoffset
#  
#FUNCEND###############################################################
function getScreenOffset () {
  local XORGCFG=${5:-$XORGCFG_DEFAULT}
  XORGCFG=${XORGCFG:-/etc/X11/xorg.conf}
  local D=$1
  cat ${XORGCFG}|\
    sed 's/"//g'|\
    awk -v d="$D" -v sect="${2//DEFAULT}" -v nr="${4//DEFAULT}" -v alias="${3//DEFAULT}" -f ${_myLIBBASE_geometry}/geometry01.awk
}

getScreenOffset $*
export -f getScreenOffset
