#!/usr/bin/bash
#
#

#
#$Header: /home2/reps/cvs/proj/ctys/ctys-rt/src/lib/libVBOXbase.sh,v 1.3 2011/12/05 15:57:43 acue Exp $
#


########################################################################
#
#PROJECT:      Unified Sessions Manager
#AUTHOR:       Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com
#MAINTAINER:   Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com
#SHORT:        ctys
#CALLFULLNAME: Commutate To Your Session
#LICENCE:      GPL3
#VERSION:      01_11_023
#
########################################################################
#
#     Copyright (C) 2010,2011 Arno-Can Uestuensoez
#
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
#
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
########################################################################



#
#
#Requires the ctys framework, at least
#
#  <bin>/bootstrap/bootstrap<version>
#  <lib>/base.h
#
#
#
#

printDBG $S_VBOX ${D_MAINT} $LINENO $BASH_SOURCE "LOAD-LIB-VBOXBASE"

#FUNCBEG###############################################################
#NAME:
#  getVersionStrgVBOX
#TYPE:
#  bash-function
#
#DESCRIPTION:
#  Fetch version info for VBOXKVM.
#
#EXAMPLE:
#
#PARAMETERS:
#  $1: name of executable
#
#OUTPUT:
#  RETURN:
#   0: OK
#   1: NOK
#  VALUES:
#
#FUNCEND###############################################################
function getVersionStrgVBOX () {
    if [ -z "${1}" ];then
	return
    fi
    _verstrg1=$(callErrOutWrapper $LINENO $BASH_SOURCE ${VBOXCALL} ${1} --help|awk '
        $0~/VirtualBox/&&$0~/Graphical/{printf("VirtualBox-%s",$NF);}
        $0~/Oracle *VM *VirtualBox/&&$0~/Manager/{printf("VirtualBox-%s",$NF);}
    ')
    echo ${_verstrg1//,/};
}



#FUNCBEG###############################################################
#NAME:
#  getVBOXMAGIC
#TYPE:
#  bash-function
#
#DESCRIPTION:
#  Fetch version info for executable of VBOX/VBOXKVM/KVM variant and
#  returns an MAGICID for machine-processing.
#
#EXAMPLE:
#
#PARAMETERS:
#  $1: name of executable
#      
#
#OUTPUT:
#  RETURN:
#   0: OK
#   1: NOK
#  VALUES:
#   Returns the value of VBOX_MAGIC, for now:
#     VBOX_030102      :VirtualBox-3.1.2
#     VBOX_GENERIC     :VirtualBox-*
#   Else
#     VBOX_NOLOCC      :*
#
#FUNCEND###############################################################
function getVBOXMAGIC () {
    printDBG $S_VBOX ${D_MAINT} $LINENO $BASH_SOURCE "\$*=<${*}>"
    local _lv=$(getVersionStrgVBOX $1)
    local _m=;
    case ${_lv} in
	VirtualBox-4.0.4*) _m=VBOX_040004;;
	VirtualBox-3.1.2)  _m=VBOX_030102;;
	VirtualBox-3.2.6*) _m=VBOX_030206;;
	VirtualBox-4.0*)   _m=VBOX_0400x;;
	VirtualBox-3.2*)   _m=VBOX_0302x;;
	VirtualBox-4.*)    _m=VBOX_04x;;
	VirtualBox-3.*)    _m=VBOX_03x;;
	VirtualBox-2*)     _m=VBOX_02x;;
	VirtualBox-*)      _m=VBOX_GENERIC;;
         *)                       echo -n NOLOC;return 1;;
    esac
    echo -n $_m
}




#FUNCBEG###############################################################
#NAME:
#  getACCELERATOR_VBOX
#TYPE:
#  bash-function
#
#DESCRIPTION:
#  Returns the available accelerator as support compiled into 
#  the tested executable.
#
#  Currently the actual availability and state of required
#  kernel modules is not checked.
#
#EXAMPLE:
#
#PARAMETERS:
#  $1: name of executable      
#
#OUTPUT:
#  RETURN:
#   0: OK
#   1: NOK
#  VALUES:
#   <accelerator>
#     VBOX
#      No specific accelerator, this value is due to SW-design.
#     ffs.
#
#FUNCEND###############################################################
function getACCELERATOR_VBOX () {
    if [ -z "${1}" ];then
	return
    fi
    local _a=$(getVBOXACCEL $1)
    echo ${_a:-VBOX}
}



#FUNCBEG###############################################################
#NAME:
#  getHYPERRELRUN_VBOX
#TYPE:
#  bash-function
#
#DESCRIPTION:
#  Returns the versionstring of current hypervisor/emulator.
#
#EXAMPLE:
#
#PARAMETERS:
#  $1: name of executable      
#
#OUTPUT:
#  RETURN:
#   0: OK
#   1: NOK
#  VALUES:
#FUNCEND###############################################################
function getHYPERRELRUN_VBOX () {
    if [ -z "${1}" ];then
	return
    fi
    case $(getACCELERATOR_VBOX $1) in
	VBOX)
	    getVersionStrgVBOX $1
	    ;;
    esac
}


#FUNCBEG###############################################################
#NAME:
#  getSTARTERCALL_VBOX4CONF
#TYPE:
#  bash-function
#
#DESCRIPTION:
#  Returns the executable from configuration file.
#  In case of multiple entries the last detected setting wins.
#
#EXAMPLE:
#
#PARAMETERS:
#  $1: name of configurationfile - *.ctys
#
#OUTPUT:
#  RETURN:
#   0: OK
#   1: NOK
#  VALUES:
#FUNCEND###############################################################
function getSTARTERCALL_VBOX4CONF () {
    if [ -z "${1}" ];then
	return
    fi
    if [ ! -f "$1" ];then
	return
    fi
    local myStarter=$(awk -F'=' 'BEGIN{x="";}/STARTERCALL/{x=$2;}END{printf("%s",x);}' ${1});
    myStarter=$(eval echo $myStarter);
    myStarter=$(bootstrapGetRealPathname ${myStarter})
    if [ -z "$myStarter" ];then
	return
    fi
    echo  $myStarter
}


#FUNCBEG###############################################################
#NAME:
#  getHYPERRELRUN_VBOX4CONF
#TYPE:
#  bash-function
#
#DESCRIPTION:
#  Returns the versionstring of current hypervisor/emulator.
#
#EXAMPLE:
#
#PARAMETERS:
#  $1: name of configurationfile - *.ctys  
#
#OUTPUT:
#  RETURN:
#   0: OK
#   1: NOK
#  VALUES:
#FUNCEND###############################################################
function getHYPERRELRUN_VBOX4CONF () {
    if [ -z "${1}" ];then
	return
    fi
    local myStarter=$(getSTARTERCALL_VBOX4CONF ${1});
    if [ -z "$myStarter" ];then
	return
    fi
    getHYPERRELRUN_VBOX $myStarter
}

#FUNCBEG###############################################################
#NAME:
#  getACCELERATOR_VBOX4CONF
#TYPE:
#  bash-function
#
#DESCRIPTION:
#  Returns the accelerator of current hypervisor/emulator.
#
#EXAMPLE:
#
#PARAMETERS:
#  $1: name of executable      
#
#OUTPUT:
#  RETURN:
#   0: OK
#   1: NOK
#  VALUES:
#FUNCEND###############################################################
function getACCELERATOR_VBOX4CONF () {
    if [ -z "${1}" ];then
	return
    fi
    local myStarter=$(getSTARTERCALL_VBOX4CONF ${1});
    if [ -z "$myStarter" ];then
	return
    fi
    getACCELERATOR_VBOX $myStarter
}


