#!/usr/bin/bash
#
#

#
#$Header: /home2/reps/cvs/proj/ctys/ctys-rt/src/plugins/HOSTs/RDP/create.sh,v 1.3 2011/12/05 15:57:44 acue Exp $
#


########################################################################
#
#PROJECT:      Unified Sessions Manager
#AUTHOR:       Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com
#MAINTAINER:   Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com
#SHORT:        ctys
#CALLFULLNAME: Commutate To Your Session
#LICENCE:      GPL3
#VERSION:      01_11_023
#
########################################################################
#
# Copyright (C) 2010,2011 Arno-Can Uestuensoez
#
########################################################################

_myPKGNAME_RDP_CREATE="${BASH_SOURCE}"
_myPKGVERS_RDP_CREATE="01.11.023"
hookInfoAdd $_myPKGNAME_RDP_CREATE $_myPKGVERS_RDP_CREATE
_myPKGBASE_RDP_CREATE="`dirname ${_myPKGNAME_RDP_CREATE}`"

export RDPHOST=;


#FUNCBEG###############################################################
#NAME:
#  createConnectRDP
#
#TYPE:
#  bash-function
#
#DESCRIPTION:
#  Creates a session and/or connects to the server.
#  
#
#EXAMPLE:
#
#PARAMETERS:
#
#
#OUTPUT:
#  RETURN:
#
#  VALUES:
#
#FUNCEND###############################################################
function createConnectRDP () {
    local OPMODE=$1;shift
    local ACTION=$1;shift

    local A;
    local KEY;
    local ARG;

    unset _RDPPORT;
    local _console=RDESKTOP;

    case ${OPMODE} in
        CHECKPARAM)
            printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "C_MODE_ARGS=$C_MODE_ARGS"
	    A=`cliSplitSubOpts ${C_MODE_ARGS}`;
            for i in $A;do                         
		KEY=`cliGetKey ${i}`
		ARG=`cliGetArg ${i}`
		case $KEY in
                    REUSE)
			local _reuse=1;
			if [ -n "${ARG}" ]; then
			    ABORT=1;
			    printERR $LINENO $BASH_SOURCE ${ABORT} "No arguments are supported for \"$KEY\""
 			    gotoHell ${ABORT}
			fi
			;;

                    CONNECT)
			local _connect=1;
			if [ -n "${ARG}" ]; then
			    ABORT=1;
			    printERR $LINENO $BASH_SOURCE ${ABORT} "No arguments are supported for \"$KEY\""
 			    gotoHell ${ABORT}
			fi
			;;

                    RECONNECT)
			local _reconnect=1;
			if [ -n "${ARG}" ]; then
			    ABORT=1;
			    printERR $LINENO $BASH_SOURCE ${ABORT} "No arguments are supported for \"$KEY\""
 			    gotoHell ${ABORT}
			fi
			;;

                    RESUME)
			ABORT=1
			printERR $LINENO $BASH_SOURCE ${ABORT} "${ACTION}:Suboption ${KEY} NOT supported"
			gotoHell ${ABORT}
			;;



        #####################
        # <machine-address> #
        #####################

                    L|LABEL)
			_LABEL="${ARG}";
			;;

                    I|ID)
			if [ -z "$_connect" ];then
			    ABORT=1
			    printERR $LINENO $BASH_SOURCE ${ABORT} "${ACTION}:Suboption ${KEYXS} NOT supported"
			    gotoHell ${ABORT}
			fi
			_ID="${_ID} ${ARG}";
			if [ -n "${ARG}" -a -n "${ARG//[0-9]/}" ];then
			    ABORT=1
			    printERR $LINENO $BASH_SOURCE ${ABORT} "Integers only:\"${ARG}\""
			    gotoHell ${ABORT}
			fi
			;;

                    STUBMODE|STUB)
			C_STUBCALL=1;
 			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "STUBMODE"
                        local _wrapper="${ARG// /}";
			case "$_wrapper" in
			    [oO][nN])C_NOWRAPPER=;
			esac
			C_CLIENTLOCATION="-L CONNECTIONFORWARDING"
 			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "WRAPPER=${_wrapper:-OFF}"
			;;



        #######################
        # Specific attributes #  
        #######################

		    CONSOLE)
			local _conty="`echo ${ARG}|tr '[:lower:]' '[:upper:]'`";
			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "_conty=${_conty}"
			case ${_conty} in
			    RDESKTOP)
				;;

			    *)
				ABORT=1;
				printERR $LINENO $BASH_SOURCE ${ABORT} "UNKNOWN ARG=${ARG}"
 				gotoHell ${ABORT}
				;;				  
			esac
			_console=$_conty;
			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "CONSOLE=${ARG}"
			;;

                    REMOTEHOST|RHOST|INSECURE)
			if [ -n "${RDPHOST}" ];then
			    ABORT=1
			    printERR $LINENO $BASH_SOURCE ${ABORT} "RDPHOST  is supported once per call only."
			    gotoHell ${ABORT}
			fi
			RDPHOST="${ARG}";
			;;


                    REMOTEPORT|RPORT)
			if [ -n "${_RDPPORT}" ];then
			    ABORT=1
			    printERR $LINENO $BASH_SOURCE ${ABORT} "RDP native port is supported once per call only"
			    gotoHell ${ABORT}
			fi
			if [ -n "${ARG}" -a -n "${ARG//[0-9]/}" ];then
			    ABORT=1
			    printERR $LINENO $BASH_SOURCE ${ABORT} "RDP native port requires integers only:\"${ARG}\""
			    gotoHell ${ABORT}
			fi
			_RDPPORT="${ARG}";
			;;

                    REMOTEBASEPORT|RBASE)
			if [ -n "${_RDPBASE}" ];then
			    ABORT=1
			    printERR $LINENO $BASH_SOURCE ${ABORT} "RDPBASE port is supported once per call only."
			    gotoHell ${ABORT}
			fi
			if [ -n "${ARG}" -a -n "${ARG//[0-9]/}" ];then
			    ABORT=1
			    printERR $LINENO $BASH_SOURCE ${ABORT} "RDPBASE port requires integers only:\"${ARG}\""
			    gotoHell ${ABORT}
			fi
			_RDPBASE="${ARG}";
			;;

		    USER)
			if [ -z "${ARG}" ];then
			    ABORT=1;
			    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing arguments:$KEY"
 			    gotoHell ${ABORT}
			fi
			_actionuserRDP="${ARG}";
			printDBG $S_VMW ${D_UID} $LINENO $BASH_SOURCE "ACTION-USER=${_actionuserRDP}"

                        local _X="${ARG//  / - }";
                        local _U="${_X%% *}";[ "${_U}" == '-' ]&&_U=;
                        local _P="${_X#* }";_P="${_P% *}";[ "${_P}" == '-' ]&&_P=;
                        local _D="${_X##* }";[ "${_D}" == '-' ]&&_D=;
			if [ -z "${_U}" ];then
			    ABORT=1;
			    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing USER:$KEY"
 			    gotoHell ${ABORT}
			fi
			if [ -z "${_P}" ];then
			    ABORT=1;
			    printINFO 2 $LINENO $BASH_SOURCE 1 "No USER-Credentials:$KEY"
			fi
			if [ -z "${_D}" ];then
			    ABORT=1;
			    printINFO 2 $LINENO $BASH_SOURCE 1 "No USER-Domain:$KEY"
			fi
			;;


		    *)
			ABORT=1
			printERR $LINENO $BASH_SOURCE ${ABORT} "Unknown suboption:<${i}>"
			printERR $LINENO $BASH_SOURCE ${ABORT} "  Probably the key is missing \"<key>:<arg>\""
			gotoHell ${ABORT}
			;;
		esac
            done
            printDBG $S_RDP ${D_MAINT} $LINENO $BASH_SOURCE "_ID      = <${_ID}>"
            printDBG $S_RDP ${D_MAINT} $LINENO $BASH_SOURCE "_LABEL   = <${_LABEL}>"
            printDBG $S_RDP ${D_MAINT} $LINENO $BASH_SOURCE "_RDP     = <${_RDP}>"
            printDBG $S_RDP ${D_MAINT} $LINENO $BASH_SOURCE "REMOTEHOST = <${RDPHOST}>"
            printDBG $S_RDP ${D_MAINT} $LINENO $BASH_SOURCE "_RDPBASE = <${_RDPBASE}>"
            printDBG $S_RDP ${D_MAINT} $LINENO $BASH_SOURCE "_RDPPORT = <${_RDPPORT}>"

	    if [ -z "${_LABEL}" -a -z "$_ID" -a -z "$R_HOSTS" -a -n "$RDPHOST" -a "$C_STUBCALL" != 1 ];then
		C_CLIENTLOCATION="-L CONNECTIONFORWARDING"
		C_STUBCALL=1;
 		printINFO 1 $LINENO $BASH_SOURCE 0 "Setting STUBMODE for RDPHOST=\"$RDPHOST\""
	    fi


	    if [ -z "${_LABEL}"  ];then
		if [ "${_connect}" == "1" ];then
		    _LABEL=${_LABEL:-DEFAULT};
		    if [ -z "${_RDPPORT}" ];then
			_RDPPORT=${_RDPPORT:-$_RDPBASE};
		    else
			if [ -n "${_ID}" ];then
			    _ID=$((RDP_BASEPORT+_ID))
			fi
		    fi
		else
		    ABORT=1;
		    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing mandatory suboption: LABEL"
		    gotoHell ${ABORT}
		fi
	    fi

            unset _ID;
            unset _LABEL;
            unset _RDPPORT;
	    ;;

	ASSEMBLE)
	    assembleExeccall
	    ;;

	PROPAGATE)
	    assembleExeccall PROPAGATE
	    ;;

	EXECUTE)
	    if [ -n "${R_TEXT}" ];then
		echo "${R_TEXT}"
	    fi
            _LABEL="";

            printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "C_MODE_ARGS=$C_MODE_ARGS"
	    A=`cliSplitSubOpts ${C_MODE_ARGS}`;
	    for i in $A;do
		KEY=`cliGetKey ${i}`
		ARG=`cliGetArg ${i}`
		case $KEY in
                    REUSE)
			local _reuse=1;
			;;

                    CONNECT)
			local _connect=1;
			;;

                    RECONNECT)
			hookPackage "${_myPKGBASE_VNC}/cancel.sh"
			local _reconnect=1;
			;;

                    RESUME)
			ABORT=1
			printERR $LINENO $BASH_SOURCE ${ABORT} "${ACTION}:Suboption ${KEY} NOT supported"
			gotoHell ${ABORT}
			;;

         #########

                    L|LABEL)
			_LABEL="${ARG}";
			;;

                    I|ID)
			_ID="${_ID} ${ARG}";
                        #Yes this call saves "a real amount" of code, even though beeing redundant itself!
  			local _IDlbl1=`fetchLabel4ID "${_ID}"`
   			local _ID1=`fetchID4Label "${_IDlbl1}"`

			if [ -n "${_ID1// /}"  -a "${_ID// /}" != "${_ID1// /}" ];then
			    ABORT=1
			    printERR $LINENO $BASH_SOURCE ${ABORT} "\"${_ID}\" seems to be ambiguous, can not be resolved"
			    printERR $LINENO $BASH_SOURCE ${ABORT} "."
			    printERR $LINENO $BASH_SOURCE ${ABORT} "  ID=\"${_ID}\" => LABEL=\"${_IDlbl1}\" => ID1=\"${_ID1}\"(first match)"
			    printERR $LINENO $BASH_SOURCE ${ABORT} "  => ID=\"${_ID}\" != ID1=\"${_ID1}\""
			    printERR $LINENO $BASH_SOURCE ${ABORT} "."
			    printERR $LINENO $BASH_SOURCE ${ABORT} "Delete ambiguous processes manually, or use \"ALL\"."
			    gotoHell ${ABORT}
			fi

   			local _IDlbl="${_IDlbl} ${_IDlbl1}"
			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "_ID=${_ID}"
			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "_IDlbl=${_IDlbl}"
			;;

                    STUBMODE|STUB)
			C_STUBCALL=1;
 			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "STUBMODE"
                        local _wrapper="${ARG// /}";
			case "$_wrapper" in
			    [oO][nN])C_NOWRAPPER=;
			esac
			C_CLIENTLOCATION="-L CONNECTIONFORWARDING"
 			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "WRAPPER=${_wrapper:-OFF}"
			;;



          #########

		    CONSOLE)
			local _conty="`echo ${ARG}|tr '[:lower:]' '[:upper:]'`";
			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "_conty=${_conty}"
			case ${_conty} in
			    RDESKTOP)
				;;

			    *)
				ABORT=1;
				printERR $LINENO $BASH_SOURCE ${ABORT} "UNKNOWN ARG=${ARG}"
 				gotoHell ${ABORT}
				;;				  
			esac
			_console=$_conty;
			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "CONSOLE=${ARG}"
			;;

                    REMOTEHOST|RHOST|INSECURE)
			RDPHOST="${ARG}";
			;;

                    REMOTEPORT|RPORT)
			_RDPPORT="${ARG}";
			printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "_RDPPORT=${_RDPPORT// /}"
			;;

                    REMOTEBASEPORT|RBASE)
			_RDPBASE="${ARG}";
			;;

		    USER)
			if [ -z "${ARG}" ];then
			    ABORT=1;
			    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing arguments:$KEY"
 			    gotoHell ${ABORT}
			fi
			_actionuserRDP="${ARG}";
			printDBG $S_VMW ${D_UID} $LINENO $BASH_SOURCE "ACTION-USER=${_actionuserRDP}"

                        local _X="${ARG//  / - }";
                        local _U="${_X%% *}";[ "${_U}" == '-' ]&&_U=;
                        local _P="${_X#* }";_P="${_P% *}";[ "${_P}" == '-' ]&&_P=;
                        local _D="${_X##* }";[ "${_D}" == '-' ]&&_D=;
			if [ -z "${_U}" ];then
			    ABORT=1;
			    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing USER:$KEY"
 			    gotoHell ${ABORT}
			fi
			if [ -z "${_P}" ];then
			    ABORT=1;
			    printINFO 2 $LINENO $BASH_SOURCE 1 "No USER-Credentials:$KEY"
			fi
			if [ -z "${_D}" ];then
			    ABORT=1;
			    printINFO 2 $LINENO $BASH_SOURCE 1 "No USER-Domain:$KEY"
			fi
			RDP_SESSION_USER=${_U};
			RDP_SESSION_CRED=${_P}
			RDP_SESSION_DOMAIN=${_D}
			RDPVIEWER="${RDPVIEWER} -u ${RDP_SESSION_USER} "
			if [ -n "${RDP_SESSION_CRED}" ];then
			    RDPVIEWER="${RDPVIEWER} -p ${RDP_SESSION_CRED} "
			else
			    RDP_SESSION_CRED=;
			fi
			if [ -n "${RDP_SESSION_DOMAIN}" ];then
			    RDPVIEWER="${RDPVIEWER} --domain ${RDP_SESSION_DOMAIN} "
			else
			    RDP_SESSION_DOMAIN=;
			fi

			printDBG $S_VMW ${D_UID} $LINENO $BASH_SOURCE "RDPVIEWER=${RDPVIEWER}"
			;;

         #########
		    *)
			ABORT=1
			printERR $LINENO $BASH_SOURCE ${ABORT} "Unknown suboption:<${i}>"
			gotoHell ${ABORT}
			;;
		esac
	    done

	    if [ -z "${_LABEL}" -a -z "$_ID" -a -z "$R_HOSTS" -a -n "$RDPHOST" -a "$C_STUBCALL" != 1 ];then
		C_CLIENTLOCATION="-L CONNECTIONFORWARDING"
		C_STUBCALL= 1
 		printINFO 1 $LINENO $BASH_SOURCE 0 "Setting STUBMODE for RDPHOST=\"$RDPHOST\""
	    fi

	    if [ -z "${_LABEL}"  ];then
		if [ "${_connect}" == "1" ];then
		    _LABEL=${_LABEL:-DEFAULT};
		    if [ -z "${_RDPPORT}" ];then
			_RDPPORT=${_RDPPORT:-$_RDPBASE};
		    else
			if [ -n "${_ID}" ];then
			    _ID=$((RDP_BASEPORT+_ID))
			fi
		    fi
		else
		    ABORT=1;
		    printERR $LINENO $BASH_SOURCE ${ABORT} "Missing mandatory suboption: LABEL"
		    gotoHell ${ABORT}
		fi
	    fi


            #perform session creation
	    printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "Start session=${_LABEL}"

            #Might be executed on remote host, not the calling station
	    printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "C_CLIENTLOCATION=${C_CLIENTLOCATION}"
 	    local _IDx=;
	    if [ -z "${_RDPPORT}" ];then
                #check for running local server
		_IDx=`fetchDisplay4Label "${_LABEL}"`
		if [ -z "${_IDx}" ];then
 		    if [ "${C_CLIENTLOCATION}" !=  "-L CONNECTIONFORWARDING" ];then
 			_IDx=${RDP_DEFAULTPORT}
		    fi
		fi
	    else
		if [ "${C_CLIENTLOCATION}" !=  "-L CONNECTIONFORWARDING" ];then
 		    _IDx=${_RDPPORT}
		fi
	    fi

	    [ -z "$_RDPPORT" ]&&_RDPPORT=${RDP_DEFAULTPORT}
	    if [ -z "${_RDPPORT}" ];then
		ABORT=1;
		printERR $LINENO $BASH_SOURCE ${ABORT} "Missing mandatory suboption: RDPPORT"
		gotoHell ${ABORT}
	    fi

	    printDBG $S_RDP ${D_MAINT} $LINENO $BASH_SOURCE " Session-Identifier: ${i}->${_ID} ${_LABEL} ->${_IDx}"
            #check whether server is already running(local or remote)
            #therfore a peer must be present
            #but nevertheless, when doing a RECONNECT on a one-shot-stub things might be different
	    if [ -n "${_IDx}" ];then
                #Server is already running, so it is a potential CONNECT.
		if [ -z "${_reconnect}" -a -z "${_connect}" -a -z "${_reuse}"  ];then
		    _connect=1;
		fi

                #for reconnect, actually for vnc only required in shared-mode,
                #this one instance replaces all active clients
		if [ -n "${_reconnect}"  ];then
		    cancelRDP CLIENT POWEROFF "i:${_IDx}"
		fi

                #perform only if REUSE is present
		if [ "${C_CLIENTLOCATION}" !=  "-L SERVERONLY" ];then
		    if [ -n "${_reconnect}" -o -n "${_connect}" -o -n "${_reuse}" ];then
			if [ -z "${_RDPPORT}" ];then
			    connectSessionRDP "${_IDx}" "${_LABEL}" "${RDPHOST}"
			else
			    connectSessionRDP "${_IDx}" "${_LABEL}" "${RDPHOST}"
			fi
		    fi
		else
		    printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "C_CLIENTLOCATION=${C_CLIENTLOCATION}"
		fi

	    else
		if [ "${C_CLIENTLOCATION}" !=  "-L CONNECTIONFORWARDING" ];then
                    #
                    #So, this is executed on server site, it is a DISPLAYFORWARDING
                    #
                    #Server not yet running, so it is a CREATE
		    if [ -n "${_connect}" -o -n "${_reconnect}"  ];then
			ABORT=1
			printERR $LINENO $BASH_SOURCE ${ABORT} "Server not available, possible causes:"
			printERR $LINENO $BASH_SOURCE ${ABORT} "- Server is not running, cannot (RE)CONNECT, use REUSE"
			printERR $LINENO $BASH_SOURCE ${ABORT} "- No access permissions, check for process ownership"
			gotoHell ${ABORT}
		    fi
		else
                    #
                    #So, this is executed on the client site, different from server site,
                    #it is CONNECTIONFORWARDING to a remote server.
                    #So, dig the tunnel and connect myself.

                    #
 		    printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "C_CLIENTLOCATION=${C_CLIENTLOCATION}"

                    #for reconnect, actually for vnc only required in shared-mode,
                    #this one instance replaces all active clients
		    if [ -n "${_reconnect}"  ];then
			cancelRDP CLIENT POWEROFF "i:${_IDx}"
		    fi

  		    _IDx=`digGetLocalPort "${_LABEL}" "RDP" "${RDPHOST}"`
                     if [ -z "$_IDx" -o -z "${_reuse}" ];then
			if [ -z "${_RDPPORT}" ];then
			    digLocalPort "RDP" "$R_HOSTS" "$_LABEL" "${RDP_DEFAULTPORT}" "" "" "${RDPHOST}" "${RDPPORT}" >/dev/null
			else
			    digLocalPort "RDP" "$R_HOSTS" "$_LABEL" "$_RDPPORT" "" "" "${RDPHOST}" >/dev/null
			fi
  			_IDx=`digGetLocalPort "${_LABEL}" "RDP" "${RDPHOST}"`
			if [ -z "$_IDx" ];then
                            #Something wrent wrong???                                      
			    ABORT=1
			    printERR $LINENO $BASH_SOURCE ${ABORT} "Cannot allocate CONNECTIONFORWARDING"
			    printERR $LINENO $BASH_SOURCE ${ABORT} "  digLocalPort <RDP> <$R_HOSTS> <$i>"
			    gotoHell ${ABORT}
			fi
		    fi
 		    printDBG $S_RDP ${D_UID} $LINENO $BASH_SOURCE "_IDx=${_IDx} _RDPPORT=${_RDPPORT}"

                    #kill of clients closes tunnel, gwhich is due to security reasons in "one-shot-mode"
		    if [ -n "${_reconnect}" -o -n "${_connect}" -o -n "${_IDx}" ];then
 			connectSessionRDP "${_IDx}" "${_LABEL}" 
		    fi			    
		fi
	    fi

	    gotoHell 0
	    ;;
    esac
}
