#!/usr/bin/bash
#
#

#
#$Header: /home2/reps/cvs/proj/ctys/ctys-rt/src/plugins/HOSTs/RDP/session.sh,v 1.3 2011/12/05 15:57:44 acue Exp $
#


########################################################################
#
#PROJECT:      Unified Sessions Manager
#AUTHOR:       Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com
#MAINTAINER:   Arno-Can Uestuensoez - unifiedsessionsmanager@protonmail.com
#SHORT:        ctys
#CALLFULLNAME: Commutate To Your Session
#LICENCE:      GPL3
#VERSION:      01_11_023
#
########################################################################
#
# Copyright (C) 2010,2011 Arno-Can Uestuensoez
#
########################################################################

_myPKGNAME_RDP_SESSION="${BASH_SOURCE}"
_myPKGVERS_RDP_SESSION="01.11.023"
hookInfoAdd $_myPKGNAME_RDP_SESSION $_myPKGVERS_RDP_SESSION
_myPKGBASE_RDP_SESSION="`dirname ${_myPKGNAME_RDP_SESSION}`"


#FUNCBEG###############################################################
#NAME:
#  getClientTPRDP
#
#TYPE:
#  bash-function
#
#DESCRIPTION:
#
# GENERIC-IF-DESCRIPTION:
#  Gives the termination points port number, to gwhich a client could be 
#  attachhed. This port is forseen to be used in port-forwarding e.g.
#  by OpenSSH.
#
#EXAMPLE:
#
#PARAMETERS:
#  <label>
#    The <label> to gwhich the client will be attached.
#
#GLOBALS:
#
#OUTPUT:
#  RETURN:
#    0: If OK
#    1: else
#
#  VALUES:
#    <TP-port>
#      The TP port, to gwhich a client could be attached.
#
#FUNCEND###############################################################
function getClientTPRDP () {
    local _arg0=${1}

    printDBG $S_RDP ${D_MAINT} $LINENO $BASH_SOURCE "$FUNCNAME Label=$_arg0"

    local _port=`fetchID4Label ${_arg0}`;
    if [ -z "${_port}" ];then
        _port=`fetchLabel4ID ${_arg0}`;
    fi
    if [ -z "${_port}" ];then
	_port=${RDP_DEFAULTPORT}
    else
 	let _port=RDP_BASEPORT+_port;    
    fi
    if [ -z "${_port}" ];then
	ABORT=2
	printINFO 2 $LINENO $BASH_SOURCE 0 "${FUNCNAME}:Cannot get port number for label:${1}"
	gotoHell ${ABORT}
    fi
    printDBG $S_RDP ${D_MAINT} $LINENO $BASH_SOURCE "$FUNCNAME port base=$RDP_BASEPORT from _port=$_port"
    echo ${_port}
}


#FUNCBEG###############################################################
#NAME:
#  connectSessionRDP
#
#TYPE:
#  bash-function
#
#DESCRIPTION:
#
#EXAMPLE:
#
#PARAMETERS:
#  $1: <session-port>
#
#  $2: <session-label>
#      This will be used for the title of the client window.
#
#  $3: [<target-host>]
#
#OUTPUT:
#  RETURN:
#
#  VALUES:
#
#FUNCEND###############################################################
function connectSessionRDP () {
    local _id0=${1}
    local _id=`removeLeadZeros ${1}`
    local _label=${2}
    local _thost=${3}
    printDBG $S_RDP ${D_MAINT} $LINENO $BASH_SOURCE "$FUNCNAME ${_id0}->${_id} ${_label}"

    if [ -z "${_label}" -a -z "${_id}" ];then
	ABORT=1
	printERR $LINENO $BASH_SOURCE ${ABORT} "${FUNCNAME}:At least one parameter required:<session-id> or <session-label>"
	gotoHell ${ABORT}
    fi

    #not-having could be tolerated, even though something might be basically wrong.
    if [ -z "${_label}" -a -n "${_id}"   ]; then 
	_label=`fetchLabel4ID ${_id}`;
	if [ -z "${_label}" ];then
	    _label=DEFAULT;
	fi
    fi

    #missing could definitely NOT be tolerated, so abort if fails.
    if [ -n "${_label}" -a -z "${_id}" ]; then 
	_id=`fetchID4Label ${_label}`;
	if [ -z "${_id}" ];then
	    ABORT=1
	    printERR $LINENO $BASH_SOURCE ${ABORT} "${FUNCNAME}:Fetch of <session-id> failed:_label=${_label}"
	    gotoHell ${ABORT}
	fi
    fi

    #even though this condition might be impossible now, let it beeeee ...
    if [ -z "${_label}" -a -z "${_id}" ];then
	ABORT=1
	printERR $LINENO $BASH_SOURCE ${ABORT} "${FUNCNAME}:Fetch of peer entry failed:_id=${_id} - _label=${_label}"
	gotoHell ${ABORT}
    fi

    printDBG $S_RDP ${D_FRAME} $LINENO $BASH_SOURCE "OK:_id=${_id} - _label=${_label}"
    #
    #Now shows name+id in title, id could not be set for server as default.
    if [ "${C_CLIENTLOCATION}" ==  "-L CONNECTIONFORWARDING" ];then
	local _vieweropt="-T ${_label}:CF:${_id} ${RDPC_OPT} ${C_GEOMETRY:+ -g $C_GEOMETRY} "
    else
	local _vieweropt="-T ${_label}:DF:${_id} ${RDPC_OPT} ${C_GEOMETRY:+ -g $C_GEOMETRY} "
    fi
    if [ -n "${_thost}" ];then
	local CALLER="${RDPVIEWER} ${C_DARGS} ${_vieweropt} ${RDPHOST}:${_id}"
    else
	local CALLER="${RDPVIEWER} ${C_DARGS} ${_vieweropt} localhost:${_id}"
    fi

    CALLER="export C_ASYNC=${C_ASYNC}&&${CALLER}  "
    printDBG $S_RDP ${D_FRAME} $LINENO $BASH_SOURCE "${CALLER}"
    export C_ASYNC;
    if [ -n "${C_DISPLAY}" ];then
	DISPLAY=":${C_DISPLAY}";
	export DISPLAY;
    fi
    printFINALCALL 0  $LINENO $BASH_SOURCE "FINAL-RDP-CONSOLE:STARTER(${_label})-DISPLAY=\"${DISPLAY}\"" "${CALLER}"

    case ${C_DISPLAY// /} in
	*[a-z][A-Z]*)
	    export DISPLAY=":$(C_SESSIONTYPE=ALL fetchDisplay4Label ALL ${C_DISPLAY})";
	    ;;
	'')
            ;;
	*)
	    export DISPLAY=":${C_DISPLAY}";
	    ;;
    esac

    if [ -z "${C_NOEXEC}" ];then
#4TEST-REMINDER-OK:	eval ${CALLER} &
	eval ${CALLER} &sleep ${CTYS_PREDETACH_TIMEOUT:-10}>/dev/null&
	sleep ${WAITS:-1}

#4TEST-TODO: 	cacheStoreWorkerPIDData CLIENT RDP "${CURSES}" "${_name}" 0 "" 
#	if [ $? -ne 0 ];then
#	    printWNG 1 $LINENO $BASH_SOURCE 0  "$FUNCNAME:RDP:Failed to store runtime JobData for CLIENT:\"${_name}\""
#	    printWNG 1 $LINENO $BASH_SOURCE 0  "$FUNCNAME:RDP: =>Restart of CLIENT should be sufficient."
#	fi
	if [ "${C_ASYNC}" == 0 ];then
	    wait
	fi
    fi
    return
}




